@@ -8,6 +8,7 @@ from cdxj_indexer.amf import amf_parse
 import base64
 import cgi
 import json
+import sys
 
 MAX_QUERY_LENGTH = 4096
 
@@ -118,7 +119,14 @@ def query_extract(mime, length, stream, url):
         try:
             query = json_parse(query_data)
         except Exception as e:
-            print(e)
+            if query_data:
+                try:
+                    sys.stderr.write(
+                        "Error parsing: " + query_data.decode("utf-8") + "\n"
+                    )
+                except:
+                    pass
+
             query = ""
 
     elif mime.startswith("text/plain"):
@@ -152,12 +160,25 @@ def json_parse(string):
         dupes[n] += 1
         return n + "." + str(dupes[n]) + "_"
 
-    def _parser(dict_var):
-        for n, v in dict_var.items():
-            if isinstance(v, dict):
-                _parser(v)
-            else:
-                data[get_key(n)] = str(v)
+    def _parser(json_obj, name=""):
+        if isinstance(json_obj, dict):
+            for n, v in json_obj.items():
+                _parser(v, n)
+
+        elif isinstance(json_obj, list):
+            for v in json_obj:
+                _parser(v, name)
+
+        elif name:
+            data[get_key(name)] = str(json_obj)
+
+    try:
+        _parser(json.loads(string))
+    except json.decoder.JSONDecodeError:
+        if b"\n" in string:
+            for line in string.split(b"\n"):
+                _parser(json.loads(line))
+        else:
+            raise
 
-    _parser(json.loads(string))
     return urlencode(data)
